"""Core abstractions for the CCProxy API."""

from ccproxy.core.async_utils import (
    async_cache_result,
    async_timer,
    gather_with_concurrency,
    get_package_dir,
    get_root_package_name,
    patched_typing,
    retry_async,
    run_in_executor,
    safe_await,
    wait_for_condition,
)
from ccproxy.core.constants import (
    ANTHROPIC_API_BASE_PATH,
    AUTH_HEADER,
    CHAT_COMPLETIONS_ENDPOINT,
    CONFIG_FILE_NAMES,
    CONTENT_TYPE_HEADER,
    CONTENT_TYPE_JSON,
    CONTENT_TYPE_STREAM,
    CONTENT_TYPE_TEXT,
    DEFAULT_DOCKER_IMAGE,
    DEFAULT_DOCKER_TIMEOUT,
    DEFAULT_MAX_TOKENS,
    DEFAULT_MODEL,
    DEFAULT_RATE_LIMIT,
    DEFAULT_STREAM,
    DEFAULT_TEMPERATURE,
    DEFAULT_TIMEOUT,
    DEFAULT_TOP_P,
    EMAIL_PATTERN,
    ENV_PREFIX,
    ERROR_MSG_INTERNAL_ERROR,
    ERROR_MSG_INVALID_REQUEST,
    ERROR_MSG_INVALID_TOKEN,
    ERROR_MSG_MODEL_NOT_FOUND,
    ERROR_MSG_RATE_LIMIT_EXCEEDED,
    JSON_EXTENSIONS,
    LOG_LEVELS,
    MAX_MESSAGE_LENGTH,
    MAX_PROMPT_LENGTH,
    MAX_TOOL_CALLS,
    MESSAGES_ENDPOINT,
    MODELS_ENDPOINT,
    OPENAI_API_BASE_PATH,
    REQUEST_ID_HEADER,
    STATUS_BAD_GATEWAY,
    STATUS_BAD_REQUEST,
    STATUS_CREATED,
    STATUS_FORBIDDEN,
    STATUS_INTERNAL_ERROR,
    STATUS_NOT_FOUND,
    STATUS_OK,
    STATUS_RATE_LIMITED,
    STATUS_SERVICE_UNAVAILABLE,
    STATUS_UNAUTHORIZED,
    STREAM_EVENT_CONTENT_BLOCK_DELTA,
    STREAM_EVENT_CONTENT_BLOCK_START,
    STREAM_EVENT_CONTENT_BLOCK_STOP,
    STREAM_EVENT_MESSAGE_DELTA,
    STREAM_EVENT_MESSAGE_START,
    STREAM_EVENT_MESSAGE_STOP,
    TOML_EXTENSIONS,
    URL_PATTERN,
    UUID_PATTERN,
    YAML_EXTENSIONS,
)
from ccproxy.core.errors import (
    MiddlewareError,
    ProxyAuthenticationError,
    ProxyConnectionError,
    ProxyError,
    ProxyTimeoutError,
    TransformationError,
)
from ccproxy.core.http import (
    BaseProxyClient,
    HTTPClient,
    HTTPConnectionError,
    HTTPError,
    HTTPTimeoutError,
    HTTPXClient,
)
from ccproxy.core.interfaces import (
    APIAdapter,
    MetricExporter,
    StreamTransformer,
    TokenStorage,
)
from ccproxy.core.interfaces import (
    RequestTransformer as IRequestTransformer,
)
from ccproxy.core.interfaces import (
    ResponseTransformer as IResponseTransformer,
)
from ccproxy.core.interfaces import (
    TransformerProtocol as ITransformerProtocol,
)
from ccproxy.core.middleware import (
    BaseMiddleware,
    CompositeMiddleware,
    MiddlewareChain,
    MiddlewareProtocol,
    NextMiddleware,
)
from ccproxy.core.proxy import (
    BaseProxy,
    HTTPProxy,
    ProxyProtocol,
    WebSocketProxy,
)
from ccproxy.core.transformers import (
    BaseTransformer,
    ChainedTransformer,
    RequestTransformer,
    ResponseTransformer,
    TransformerProtocol,
)
from ccproxy.core.types import (
    MiddlewareConfig,
    ProxyConfig,
    ProxyMethod,
    ProxyRequest,
    ProxyResponse,
    TransformContext,
)
from ccproxy.core.types import (
    ProxyProtocol as ProxyProtocolEnum,
)
from ccproxy.core.validators import (
    ValidationError,
    validate_choice,
    validate_dict,
    validate_email,
    validate_list,
    validate_non_empty_string,
    validate_path,
    validate_port,
    validate_range,
    validate_timeout,
    validate_url,
    validate_uuid,
)


__all__ = [
    # Proxy abstractions
    "BaseProxy",
    "HTTPProxy",
    "WebSocketProxy",
    "ProxyProtocol",
    # HTTP client abstractions
    "HTTPClient",
    "BaseProxyClient",
    "HTTPError",
    "HTTPTimeoutError",
    "HTTPConnectionError",
    "HTTPXClient",
    # Interface abstractions
    "APIAdapter",
    "MetricExporter",
    "IRequestTransformer",
    "IResponseTransformer",
    "StreamTransformer",
    "TokenStorage",
    "ITransformerProtocol",
    # Transformer abstractions
    "BaseTransformer",
    "RequestTransformer",
    "ResponseTransformer",
    "TransformerProtocol",
    "ChainedTransformer",
    # Middleware abstractions
    "BaseMiddleware",
    "MiddlewareChain",
    "MiddlewareProtocol",
    "CompositeMiddleware",
    "NextMiddleware",
    # Error types
    "ProxyError",
    "TransformationError",
    "MiddlewareError",
    "ProxyConnectionError",
    "ProxyTimeoutError",
    "ProxyAuthenticationError",
    "ValidationError",
    # Type definitions
    "ProxyRequest",
    "ProxyResponse",
    "TransformContext",
    "ProxyMethod",
    "ProxyProtocolEnum",
    "ProxyConfig",
    "MiddlewareConfig",
    # Async utilities
    "async_cache_result",
    "async_timer",
    "gather_with_concurrency",
    "get_package_dir",
    "get_root_package_name",
    "patched_typing",
    "retry_async",
    "run_in_executor",
    "safe_await",
    "wait_for_condition",
    # Constants
    "ANTHROPIC_API_BASE_PATH",
    "AUTH_HEADER",
    "CHAT_COMPLETIONS_ENDPOINT",
    "CONFIG_FILE_NAMES",
    "CONTENT_TYPE_HEADER",
    "CONTENT_TYPE_JSON",
    "CONTENT_TYPE_STREAM",
    "CONTENT_TYPE_TEXT",
    "DEFAULT_DOCKER_IMAGE",
    "DEFAULT_DOCKER_TIMEOUT",
    "DEFAULT_MAX_TOKENS",
    "DEFAULT_MODEL",
    "DEFAULT_RATE_LIMIT",
    "DEFAULT_STREAM",
    "DEFAULT_TEMPERATURE",
    "DEFAULT_TIMEOUT",
    "DEFAULT_TOP_P",
    "EMAIL_PATTERN",
    "ENV_PREFIX",
    "ERROR_MSG_INTERNAL_ERROR",
    "ERROR_MSG_INVALID_REQUEST",
    "ERROR_MSG_INVALID_TOKEN",
    "ERROR_MSG_MODEL_NOT_FOUND",
    "ERROR_MSG_RATE_LIMIT_EXCEEDED",
    "JSON_EXTENSIONS",
    "LOG_LEVELS",
    "MAX_MESSAGE_LENGTH",
    "MAX_PROMPT_LENGTH",
    "MAX_TOOL_CALLS",
    "MESSAGES_ENDPOINT",
    "MODELS_ENDPOINT",
    "OPENAI_API_BASE_PATH",
    "REQUEST_ID_HEADER",
    "STATUS_BAD_GATEWAY",
    "STATUS_BAD_REQUEST",
    "STATUS_CREATED",
    "STATUS_FORBIDDEN",
    "STATUS_INTERNAL_ERROR",
    "STATUS_NOT_FOUND",
    "STATUS_OK",
    "STATUS_RATE_LIMITED",
    "STATUS_SERVICE_UNAVAILABLE",
    "STATUS_UNAUTHORIZED",
    "STREAM_EVENT_CONTENT_BLOCK_DELTA",
    "STREAM_EVENT_CONTENT_BLOCK_START",
    "STREAM_EVENT_CONTENT_BLOCK_STOP",
    "STREAM_EVENT_MESSAGE_DELTA",
    "STREAM_EVENT_MESSAGE_START",
    "STREAM_EVENT_MESSAGE_STOP",
    "TOML_EXTENSIONS",
    "URL_PATTERN",
    "UUID_PATTERN",
    "YAML_EXTENSIONS",
    # Validators
    "validate_choice",
    "validate_dict",
    "validate_email",
    "validate_list",
    "validate_non_empty_string",
    "validate_path",
    "validate_port",
    "validate_range",
    "validate_timeout",
    "validate_url",
    "validate_uuid",
]
